"use strict";

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.findDifference = exports.required = undefined;
exports.validation = validation;
exports.findErrors = findErrors;
exports.displayError = displayError;
exports.ifArg = ifArg;
exports.toJSON = toJSON;
exports.beautifyMessages = beautifyMessages;
exports.objectLength = objectLength;
exports.relativePath = relativePath;
exports.checkExistLocaleByName = checkExistLocaleByName;
exports.existLocale = existLocale;
exports.checkFileExists = checkFileExists;

var _ramda = require("ramda");

var _ramda2 = _interopRequireDefault(_ramda);

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _fs = require("fs");

var _jsBeautify = require("js-beautify");

var _jsBeautify2 = _interopRequireDefault(_jsBeautify);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
 * Return a missing parameter error when the user did not mention the parameters in the function call.
 * @param  {String} parameter Parameter name.
 * @return {Function}         Return and throw an missing Error.
 */
var required = exports.required = function required(parameter) {
	throw new Error("Missing parameter: " + parameter);
};

/**
 * findDifference - Find diffrence of two object with together.
 * @param  {Object} locales
 * @param  {Object} all
 * @return {Object}
 */
var findDifference = exports.findDifference = function findDifference(locales, diffs) {
	return _ramda2.default.filter(function (diff) {
		return !_ramda2.default.has(diff)(locales) && !_ramda2.default.eqProps(diff, diffs, locales);
	}, _ramda2.default.keys(diffs)).map(function (diff) {
		return _defineProperty({}, diff, diffs[diff]);
	}).reduce(_ramda2.default.merge, {});
};

/**
 * Validate all keys which that the intl-combiner need to use.
 * @param  {Array} locales  The list of all locales for use.
 * @param  {String} diffPath Path to save the diff file.
 * @param  {String} messages Path to all messages with glob method => ./src/ ** /messages.js
 * @param  {String} findBy   The goal locale which does you want to intl-combiner works on that like en or fa [this name should be the name of locale file in your locales list]
 * If you have 2 locales with this model:
 * [
 * 		{name: "en", path: "./src/locales/en.js"},
 * 		{name: "fr", path: "./src/locales/fr.js"}
 * ]
 * So, if you want to work on en.js, then you must set the "en" for this property by terminal or .combinerrc file.
 * @return {Boolean}          If there are no any errors the it will be returned true and vice versa.
 */
function validation(_ref2) {
	var locales = _ref2.locales,
	    diffPath = _ref2.diffPath,
	    messages = _ref2.messages,
	    findBy = _ref2.findBy;

	return _ramda2.default.length(locales) > 0 && !_ramda2.default.isEmpty(messages) && !_ramda2.default.isEmpty(findBy);
}

/**
 * Get the list of errors which you have in your config or terminal commands.
 * All properties is the same of the validation function properties detailed.
 */
function findErrors(_ref3) {
	var locales = _ref3.locales,
	    diffPath = _ref3.diffPath,
	    messages = _ref3.messages,
	    findBy = _ref3.findBy;

	var errors = [];

	_ramda2.default.length(locales) === 0 && errors.push("locales");
	_ramda2.default.isEmpty(messages) && errors.push("messages");
	_ramda2.default.isEmpty(findBy) && errors.push("findBy");

	return errors;
}

/**
 * Display errors in the terminal.
 * @param  {Array<String>} errors The list of errors names.
 */
function displayError(errors) {
	console.error("\n\x1B[1m\x1B[31mInsufficient number of arguments" + (!_ramda2.default.isEmpty(errors) ? " or no " + _ramda2.default.join(" and ")(errors) : "") + " found.");
	console.error("\x1B[1m\x1B[31mAlternatively, run 'intl-combiner --help' for usage info.\x1B[39m\x1B[22m\n");
	process.exit(1);
}

/**
 * Check if the goal argument was used by the user in the terminal or not.
 * @param  {Array<Function>} argv
 * @return {Function}
 */
function ifArg(argv) {
	return function (name, fn, init) {
		if (Array.isArray(argv[name])) {
			if (init) init();
			argv[name].forEach(fn);
		} else if (typeof argv[name] !== "undefined") {
			if (init) init();
			fn(argv[name], -1);
		}
	};
}

/**
 * toJSON - Parse JSON as Sting
 * @return {Object} Parsed JSON
 */
function toJSON(ObjectStr) {
	return JSON.parse(ObjectStr);
}

/**
 * beautifyMessages - Beautify rendered json file as indent and json styles.
 * @return {Object} Beautified json string.
 */
function beautifyMessages(messages) {
	return _jsBeautify2.default.js(JSON.stringify(messages), {
		indent_size: "4",
		indent_char: " ",
		max_preserve_newlines: "5",
		preserve_newlines: true,
		keep_array_indentation: false,
		break_chained_methods: false,
		indent_scripts: "normal",
		brace_style: "collapse",
		space_before_conditional: true,
		unescape_strings: false,
		jslint_happy: false,
		end_with_newline: false,
		wrap_line_length: "0",
		indent_inner_html: false,
		comma_first: false,
		e4x: false
	});
}
/**
 * Return the object length
 * @param  {Object} object
 * @return {Number}        The length of object
 */
function objectLength(object) {
	return _ramda2.default.length(_ramda2.default.keys(object));
}

function relativePath(pth) {
	return _path2.default.relative(process.cwd(), pth);
}

function checkExistLocaleByName(name, locales) {
	return !!_ramda2.default.find(_ramda2.default.propEq("name", name))(locales);
}

function existLocale(name, locales) {
	if (checkExistLocaleByName(name, locales)) {
		return true;
	} else {
		console.log(_chalk2.default.white.bgRed.bold("The " + name + ".json file doesn't exist in your locales folder. please check it again."));

		process.exitCode = 2;

		return false;
	}
}

function checkFileExists(list) {
	var notFoundList = [];

	_ramda2.default.forEach(function (_ref4) {
		var name = _ref4.name,
		    path = _ref4.path;
		return !(0, _fs.existsSync)(path) && notFoundList.push(name + " => " + relativePath(path));
	})(list);

	return notFoundList;
}